--- loveui is a love2d library to provide resuable GUI widgets to love2d 
-- developers.

module ("ui", package.seeall)

polygon = love.graphics.polygon
rectangle = love.graphics.rectangle
push = love.graphics.push
pop = love.graphics.pop
rotate = love.graphics.rotate
translate = love.graphics.translate
scale = love.graphics.scale
color = love.graphics.setColor
line = love.graphics.line
draw = love.graphics.draw
framebuffer = love.graphics.newFramebuffer
rendertarget = love.graphics.setRenderTarget

--- Draw a sub-part of a circle.
-- TODO: Draw antialiased arc.
-- @param x The center x coordinate.
-- @param y The center y coordinate.
-- @param angle The angle to rotate the arc.
-- @param radius The radius from the outer side of the arc.
-- @param width The thickness of the arc.
-- @param length The length of the arc in radians.
function arc(mode, x, y, angle, radius, width, length)
  assert(mode == "fill" or mode == "line")
  -- Width cannot be larger than radius.
  width = math.min(radius, width)
  
  -- Angle must be between 0 and 2 PI
  angle = math.max(math.min(math.pi * 2, angle), 0)
  
  -- Precision of the round edge.
  local delta = 0.01*math.pi
  
  -- Trigonometry
  local function pointx(radius, angle)
    return radius*math.cos(angle)+x
  end
  local function pointy(radius, angle)
    return radius*math.sin(angle)+y
  end
  
  local last = angle
  love.graphics.setLine( 1, "smooth" )
  -- Draw arc by drawing many little trapezoids.
  
  -- Modify the radius to produce anti-alias effect.
  local outer = radius-0.5
  local inner = radius-width+0.5
  
  local function trapezoid(last, current)
    if mode == "fill" then
      polygon("fill",
        pointx(inner, last), pointy(inner, last),
        pointx(outer, last), pointy(outer, last),
        pointx(outer, current), pointy(outer, current),
        pointx(inner, current),pointy(inner, current))
    end
    -- Anti-aliasing round edges of arc.
    line(pointx(outer, last), pointy(outer, last), 
      pointx(outer, current), pointy(outer, current))
    
    line(pointx(inner, last), pointy(inner, last),
      pointx(inner, current), pointy(inner, current))
  end
  
  for current = angle+delta, angle+length, delta do
    trapezoid(last, current)
    last = current
  end
  trapezoid(last, angle+length)
  
  -- Anti-aliasing flat edges of arc.
  line(pointx(outer, angle), pointy(outer, angle),
    pointx(inner, angle), pointy(inner, angle))
  line(pointx(outer, angle+length), pointy(outer, angle+length),
    pointx(inner, angle+length), pointy(inner, angle+length))
end
--[[
local oldarc, arcs = arc, {}
function arc(mode, x, y, angle, radius, width, length)
  local hash = table.concat({mode, x, y, angle, radius, width, length}, "|")
  if not arcs[hash] then
    local fb = framebuffer(radius, radius) -- convert to power of 2?
    rendertarget(fb)
    oldarc(mode, x, y, angle, radius, width, length)
    rendertarget()
    arcs[hash] = love.graphics.newImage(fb:getImageData())
  end
  draw(arcs[hash], x - radius, y - radius)
end
]]--